import re

import requests


def search_crtsh(self, target):
  print("Searching crt.sh")
  headers = {'Pragma':          'no-cache', 'DNT': '1',
             'Accept-Encoding': 'gzip, deflate, br',
             'Accept-Language': 'en-US,en;q=0.9,it;q=0.8',
             'User-Agent':      'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/63.0.3239.84 Safari/537.36',
             'Accept':          'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8',
             'Cache-Control':   'no-cache', 'Connection': 'keep-alive', }

  params = (('q', '%.' + target),)

  res = requests.get('https://crt.sh/', headers=headers, params=params)
  try:
    scraped = res.text
    subdomain_finder = re.compile('<TD>(.*\.' + target + ')</TD>')
    links = subdomain_finder.findall(scraped)
    for domain in links:
      if domain.strip() not in self.domains and domain.endswith("." + target):
        self.domains.append(domain.strip())
        if self.options["--verbose"]:
          print("Crt.sh Found Domain:", domain.strip())
  except Exception as e:
    self.handle_exception(e, "Error searching crt.sh")



import requests


def search_pkey(self, target):
  print("Searching Pkey.in")
  headers = {'Pragma':          'no-cache', 'Origin': 'https://www.pkey.in',
             'Accept-Encoding': 'gzip, deflate, br',
             'Accept-Language': 'en-US,en;q=0.9,it;q=0.8',
             'User-Agent':      'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/62.0.3202.94 Safari/537.36',
             'Content-Type':    'application/x-www-form-urlencoded',
             'Accept':          'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8',
             'Cache-Control':   'no-cache',
             'Referer':         'http://www.pkey.in/tools-i/search-subdomains',
             'Connection':      'keep-alive', 'DNT': '1', }

  data = [('zone', target), ('submit', ''), ]
  res = requests.post('https://www.pkey.in/tools-i/search-subdomains',
                      headers=headers, data=data, verify=False)
  scraped = res.text
  try:
    trimmed = scraped[scraped.find('<table class="clearborder">'):scraped.rfind(
      '</tbody>')].split('\n')
    for entry in trimmed:
      if entry.strip().startswith('<td style="border-left-style: none;">'):
        domain = entry.replace('<td style="border-left-style: none;">', '')
        domain = domain.replace('</td>', '')
        if target in domain:
          if domain.strip() not in self.domains:
            self.domains.append(domain.strip())
            if self.options["--verbose"]:
              print("Pkey Found Domain:", domain.strip())
              
  except Exception as e:
    self.handle_exception(e, "Error parsing pkey")
    pass



from sslyze.plugins.certificate_info_plugin import CertificateInfoScanCommand
from sslyze.plugins.heartbleed_plugin import HeartbleedScanCommand
from sslyze.plugins.http_headers_plugin import HttpHeadersScanCommand
from sslyze.plugins.openssl_cipher_suites_plugin import Tlsv10ScanCommand, Tlsv12ScanCommand
from sslyze.server_connectivity import ServerConnectivityInfo
from sslyze.synchronous_scanner import SynchronousScanner


def search_subject_alt_name(self, target):
  print("Searching for Subject Alt Names")
  try:
    server_info = ServerConnectivityInfo(hostname=target)
    server_info.test_connectivity_to_server()
    synchronous_scanner = SynchronousScanner()

    # Certificate information
    command = CertificateInfoScanCommand()
    scan_result = synchronous_scanner.run_scan_command(server_info, command)
    # Direct object reference is pretty bad, but then again so is the crypto.x509 object implementation, so...
    extensions = scan_result.certificate_chain[0].extensions[6]
    for entry in extensions.value:
      if entry.value.strip() not in self.domains:
        self.domains.append(entry.value.strip())

  except Exception as e:
    self.handle_exception(e)


def ssl_scan(self, target):
  print("Running SSL Scan")
  try:
    server_info = ServerConnectivityInfo(hostname=target)
    server_info.test_connectivity_to_server()
    synchronous_scanner = SynchronousScanner()

    # TLS 1.0
    command = Tlsv10ScanCommand()
    scan_result = synchronous_scanner.run_scan_command(server_info, command)
    print("Available TLSv1.0 Ciphers:")
    for cipher in scan_result.accepted_cipher_list:
      print('    {}'.format(cipher.name))

    # TLSv1.2
    command = Tlsv12ScanCommand()
    scan_result = synchronous_scanner.run_scan_command(server_info, command)
    print("Available TLSv1.2 Ciphers:")
    for cipher in scan_result.accepted_cipher_list:
      print('    {}'.format(cipher.name))

    # Certificate information
    command = CertificateInfoScanCommand()
    scan_result = synchronous_scanner.run_scan_command(server_info, command)
    for entry in scan_result.as_text():
      print(entry)

    # Heartbleed vulnerability info
    command = HeartbleedScanCommand()
    scan_result = synchronous_scanner.run_scan_command(server_info, command)
    for entry in scan_result.as_text():
      print(entry)

    # HTTP Headers info
    command = HttpHeadersScanCommand()
    scan_result = synchronous_scanner.run_scan_command(server_info, command)
    for entry in scan_result.as_text():
      print(entry)

  except Exception as e:
    self.handle_exception(e, "Error running SSL scan")
    pass



"""The target command."""
import queue
import re
import signal
import socket
import sys
import threading
from io import StringIO
from threading import Thread
from urllib.parse import urlsplit

from anubis.scanners.anubis_db import search_anubisdb, send_to_anubisdb
from anubis.scanners.brute_force import brute_force
from anubis.scanners.crt import search_crtsh
from anubis.scanners.dnsdumpster import search_dnsdumpster
from anubis.scanners.dnssec import dnssecc_subdomain_enum
from anubis.scanners.hackertarget import subdomain_hackertarget
from anubis.scanners.netcraft import search_netcraft
from anubis.scanners.nmap import scan_host
from anubis.scanners.pkey import search_pkey
from anubis.scanners.shodan import search_shodan
from anubis.scanners.ssl import search_subject_alt_name, ssl_scan
from anubis.scanners.virustotal import search_virustotal
from anubis.scanners.zonetransfer import dns_zonetransfer
from anubis.utils.ColorPrint import ColorPrint
from anubis.utils.search_worker import SearchWorker
from anubis.utils.signal_handler import SignalHandler
from .base import Base


class Target(Base):
  """Main enumeration module"""
  domains = list()
  ip = str()
  dedupe = set()

  stdout = sys.stdout

  def handle_exception(self, e, message=""):
    if self.options["--verbose"]:
      print(e)
    if message:
      ColorPrint.red(message)

  def init(self):
    self.options["TARGET"] = self.options["TARGET"].split(",")
    for i in range(len(self.options["TARGET"])):
      url = self.options["TARGET"][i]

      if not re.match(r'http(s?):', url):
        url = 'http://' + url

      parsed = urlsplit(url)
      host = parsed.netloc

      if host.startswith('www.'):
        host = host[4:]

      self.options["TARGET"][i] = host

      try:
        ColorPrint.green("Searching for subdomains for " + socket.gethostbyname(
          self.options["TARGET"][i]) + " (" + self.options["TARGET"][i] + ")")
      except Exception as e:
        self.handle_exception(e,
                              "Error connecting to target! Make sure you spelled it correctly and it is a resolvable address")
        raise e
    print("")

  def run(self):
    # Retrieve IP of target and run initial configurations
    self.init()
    for i in range(len(self.options["TARGET"])):
      # Default scans that run every time
      threads = [
        Thread(target=dns_zonetransfer(self, self.options["TARGET"][i])),
        Thread(target=search_subject_alt_name(self, self.options["TARGET"][i])),
        Thread(target=subdomain_hackertarget(self, self.options["TARGET"][i])),
        Thread(target=search_virustotal(self, self.options["TARGET"][i])),
        Thread(target=search_pkey(self, self.options["TARGET"][i])),
        Thread(target=search_netcraft(self, self.options["TARGET"][i])),
        Thread(target=search_crtsh(self, self.options["TARGET"][i])),
        Thread(target=search_dnsdumpster(self, self.options["TARGET"][i])),
        Thread(target=search_anubisdb(self, self.options["TARGET"][i]))]
      # Additional options - ssl cert scan
      if self.options["--ssl"]:
        threads.append(Thread(target=ssl_scan(self, self.options["TARGET"][i])))

      # Additional options - shodan.io scan
      if self.options["--additional-info"]:
        threads.append(Thread(target=search_shodan(self)))

      # Additional options - nmap scan of dnssec script and a host/port scan
      if self.options["--with-nmap"]:
        threads.append(Thread(
          target=dnssecc_subdomain_enum(self, self.options["TARGET"][i])))
        threads.append(Thread(target=scan_host(self)))

      # Additional options - brute force common subdomains
      if self.options["--brute-force"]:
        threads.append(
          Thread(target=brute_force(self, self.options["TARGET"][i])))

      # Start all threads
    for x in threads:
      x.start()

    # Wait for all of them to finish
    for x in threads:
      x.join()

    # remove duplicates and clean up

    if self.options["--recursive"]:
      self.recursive_search()

    self.domains = self.clean_domains(self.domains)
    self.dedupe = set(self.domains)

    print("Found", len(self.dedupe), "subdomains")
    print("----------------")

    if self.options["--ip"]:
      self.resolve_ips()
    else:
      for domain in self.dedupe:
        ColorPrint.green(domain.strip())

    if not self.options["--no-anubis-db"]:
      send_to_anubisdb(self, self.options["TARGET"])

  def clean_domains(self, domains):
    cleaned = []
    for subdomain in domains:
      subdomain = subdomain.lower()
      if subdomain.find("//") != -1:
        subdomain = subdomain[subdomain.find("//") + 2:]
      # Some pkey return instances like example.com. - remove the final .
      if subdomain.endswith('.'):
        subdomain = subdomain[:-1]
      # sometimes we'll get something like /www.example.com
      if subdomain[0] in ["\\", ".", "/", "#", "$", "%"]:
        subdomain = subdomain[1:]
      # If it's an email address, only take the domain part
      if "@" in subdomain:
        subdomain = subdomain.split("@")
        # If it's an actual email like mail@example.com, take example.com
        if len(subdomain) > 1:
          subdomain = subdomain[1]
        else:
          # If for some reason it's example.com@, take example.com
          subdomain = subdomain[0]

      cleaned.append(subdomain.strip())
    return cleaned

  def resolve_ips(self):
    unique_ips = set()
    for domain in self.dedupe:
      try:
        # Attempt to get IP
        resolved_ip = socket.gethostbyname(domain)
      except Exception as e:
        self.handle_exception(e)
        # If getting IP fails, fallback to empty string
        resolved_ip = ""
      # TODO - Align domains and ips in stdout
      ColorPrint.green(domain + ": " + resolved_ip)
      unique_ips.add(resolved_ip)
    print("Found %s unique IPs" % len(unique_ips))
    for ip in unique_ips:
      # String truthiness ignores empty strings
      if ip:
        ColorPrint.green(ip)

  def recursive_search(self):
    print("Starting recursive search - warning, might take a long time")
    domains = self.clean_domains(self.domains)
    domains_unique = set(domains)

    num_workers = 10

    if self.options["--queue-workers"]:
      num_workers = int(self.options["--queue-workers"])

    stopper = threading.Event()
    url_queue = queue.Queue()
    for domain in domains_unique:
      url_queue.put(domain)

    # we need to keep track of the workers but not start them yet
    workers = [SearchWorker(url_queue, self.domains, stopper, self) for _ in
               range(num_workers)]

    # create our signal handler and connect it
    handler = SignalHandler(stopper, workers)
    signal.signal(signal.SIGINT, handler)

    if not self.options["--verbose"]:
      # catch stdout and replace it with our own
      self.stdout, sys.stdout = sys.stdout, StringIO()

    # start the threads!
    for worker in workers:
      worker.start()

    # wait for the queue to empty
    url_queue.join()

    sys.stdout = self.stdout



"""Search worker """

import queue
import sys
import threading
from threading import Thread

from anubis.scanners.crt import search_crtsh
from anubis.scanners.dnsdumpster import search_dnsdumpster
from anubis.scanners.hackertarget import subdomain_hackertarget
from anubis.scanners.netcraft import search_netcraft
from anubis.scanners.pkey import search_pkey
from anubis.scanners.ssl import search_subject_alt_name
from anubis.scanners.virustotal import search_virustotal
from anubis.scanners.zonetransfer import dns_zonetransfer


class SearchWorker(threading.Thread):
  """
  The thread that will check HTTP statuses.
  """

  #: The queue of urls
  domain_queue = None

  #: An event that tells the thread to stop
  stopper = None

  domains = list()
  master_domains = None
  parent = None

  def __init__(self, domain_queue, domains, stopper, parent):
    super().__init__()
    self.domain_queue = domain_queue
    self.stopper = stopper
    self.master_domains = domains
    self.parent = parent

  def run(self):
    while not self.stopper.is_set():
      try:
        target = self.domain_queue.get_nowait()
      except queue.Empty:
        break
      else:
        sys.__stdout__.write("Starting recursive search on " + target + "\n")
        self.parent.stdout.flush()
        # Default scans that run every time
        threads = [Thread(target=dns_zonetransfer(self.parent, target)),
                   Thread(target=search_subject_alt_name(self.parent, target)),
                   Thread(target=subdomain_hackertarget(self.parent, target)),
                   Thread(target=search_virustotal(self.parent, target)),
                   Thread(target=search_pkey(self.parent, target)),
                   Thread(target=search_netcraft(self.parent, target)),
                   Thread(target=search_crtsh(self.parent, target)),
                   Thread(target=search_dnsdumpster(self.parent, target))]

        # Start all threads
        for x in threads:
          x.start()

        # Wait for all of them to finish
        for x in threads:
          x.join()

        self.domains = self.parent.clean_domains(self.domains)
        for domain in self.domains:
          if domain not in self.master_domains:
            sys.__stdout__.write("Found new domain: " + domain)
            self.master_domains.append(domain)
            self.domain_queue.put(domain)

        self.domain_queue.task_done()



"""Tests for our main anubis CLI module."""

import os
import shutil
import tempfile
from subprocess import PIPE, Popen as popen
from unittest import TestCase

from anubis import __version__ as VERSION
from anubis.cli import StdOutHook


class TestHelp(TestCase):

  def test_returns_usage_information(self):
    output = popen(['anubis', '-h'], stdout=PIPE).communicate()[0]
    output = output.decode("utf-8")
    self.assertTrue('Usage:' in output)

    output = popen(['anubis', '--help'], stdout=PIPE).communicate()[0]
    output = output.decode("utf-8")
    self.assertTrue('Usage:' in output)


class TestVersion(TestCase):

  def test_returns_version_information(self):
    output = popen(['anubis', '--version'], stdout=PIPE).communicate()[0]
    output = output.decode("utf-8")
    self.assertEqual(output.strip(), VERSION)


class TestOptions(TestCase):

  def setUp(self):
    # Create a temporary directory
    self.test_dir = tempfile.mkdtemp()

  def tearDown(self):
    # Remove the directory after the test
    shutil.rmtree(self.test_dir)

  def test_stdouthook(self):
    temp = StdOutHook(os.path.join(self.test_dir, 'test.txt'))
    temp.write("test")
    temp.write_out()
    temp.flush()
    f = open(os.path.join(self.test_dir, 'test.txt'), 'r')
    self.assertIn("test", f.readlines())



"""Tests for our main anubis CLI module."""

import os
import sys
from io import StringIO
from unittest import TestCase

from anubis.scanners.anubis_db import search_anubisdb, send_to_anubisdb
from anubis.scanners.brute_force import brute_force
from anubis.scanners.crt import search_crtsh
from anubis.scanners.dnsdumpster import search_dnsdumpster
from anubis.scanners.dnssec import dnssecc_subdomain_enum
from anubis.scanners.hackertarget import subdomain_hackertarget
from anubis.scanners.netcraft import search_netcraft
from anubis.scanners.pkey import search_pkey
from anubis.scanners.shodan import search_shodan
from anubis.scanners.ssl import search_subject_alt_name, ssl_scan
from anubis.scanners.virustotal import search_virustotal
from anubis.scanners.zonetransfer import dns_zonetransfer
from anubis.utils.ColorPrint import ColorPrint


class TestScanners(TestCase):
  domains = list()
  options = {"--verbose": True}

  def handle_exception(self, e, message=""):
    print(e)
    if message:
      print(message)

  def setUp(self):
    # catch stdout
    self.held, sys.stdout = sys.stdout, StringIO()
    # reset domains
    self.domains = list()

  def tearDown(self):
    self.held.write(sys.stdout.getvalue())

  def test_anubis_db(self):
    search_anubisdb(self, "example.com")
    self.assertIn("www.example.com", self.domains)

  def test_send_to_anubis(self):
    self.domains.append("www.example.com")
    send_to_anubisdb(self,
                     ["example.com"])  # Send to anubis db takes in an array
    self.assertTrue("Error" not in sys.stdout.getvalue())

  def test_crt(self):
    search_crtsh(self, "jonlu.ca")
    self.assertIn("secure.jonlu.ca", self.domains)

  def test_bruteforce(self):
    brute_force(self, "jonlu.ca")
    # TODO implement bruteforce tests
    self.assertTrue(True)

  def test_zonetransfer(self):
    dns_zonetransfer(self, "jonlu.ca")
    self.assertTrue("Error" not in sys.stdout.getvalue())
    dns_zonetransfer(self, "$FALSE$")
    self.assertTrue("Error" in sys.stdout.getvalue())

  def test_virustotal(self):
    search_virustotal(self, "example.com")
    if "limiting" in sys.stdout.getvalue():
      print("VirusTotal rate limiting")
      return
    self.assertIn("yy.example.com", self.domains)

  def test_dnsdumpster(self):
    search_dnsdumpster(self, "example.com")
    self.assertIn("www.example.com", self.domains)

  def test_dnssec(self):
    if os.getuid() == 0:
      dnssecc_subdomain_enum(self, "google.com")
      self.assertTrue("google" in sys.stdout.getvalue())
    else:
      print("To run DNSSEC test, run as root")

  def test_hackertarget(self):
    subdomain_hackertarget(self, "example.com")
    self.assertIn("www.example.com", self.domains)

  def test_netcraft(self):
    search_netcraft(self, "example.com")
    self.assertIn("http://www.example.com", self.domains)

  def test_pkey(self):
    search_pkey(self, "google.com")
    self.assertIn("google.com", self.domains)

  def test_shodan(self):
    self.ip = "138.197.125.24"
    self.options = {}
    self.options["TARGET"] = ["jonlu.ca"]
    self.options["--verbose"] = True
    search_shodan(self)
    self.assertIn("ISP", sys.stdout.getvalue())

  def test_ssl(self):
    ssl_scan(self, "jonlu.ca")
    # Check that Lets Encrypt Cert id is in output
    self.assertIn("YLh1dUR9y6Kja30RrAn7JKnbQG/uEtLMkBgFF2Fuihg=",
                  sys.stdout.getvalue())

  def test_san(self):
    ssl_scan(self, "jonlu.ca")
    search_subject_alt_name(self, "jonlu.ca")
    self.assertIn("www.jonlu.ca", self.domains)


class TestColorPrint(TestCase):

  def setUp(self):
    # catch stdout
    self.held, sys.stdout = sys.stdout, StringIO()
    # reset domains
    self.domains = list()

  def tearDown(self):
    # Kind of hacky as it'll dump all of stdout at the end, but better than losing it entirely
    self.held.write(sys.stdout.getvalue())

  def test_color_print(self):
    ColorPrint.red("red")
    self.assertIn("91m", sys.stdout.getvalue())
    ColorPrint.green("green")
    self.assertIn("92m", sys.stdout.getvalue())
    ColorPrint.light_purple("light_purple")
    self.assertIn("94m", sys.stdout.getvalue())
    ColorPrint.purple("purple")
    self.assertIn("95m", sys.stdout.getvalue())
    ColorPrint.yellow("yellow")
    self.assertIn("93m", sys.stdout.getvalue())



import requests


def subdomain_hackertarget(self, target):
  print("Searching HackerTarget")
  headers = {
    'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/62.0.3202.94 Safari/537.36', }
  params = (('q', target),)

  results = requests.get('http://api.hackertarget.com/hostsearch/',
                         headers=headers, params=params)
  results = results.text.split('\n')
  for res in results:
    try:
      if res.split(",")[0] != "":
        domain = res.split(",")[0]
        domain = domain.strip()
        if domain not in self.domains and domain.endswith("." + target):
          self.domains.append(domain)
          if self.options["--verbose"]:
            print("HackerTarget Found Domain:", domain.strip())
    except:
      pass



import requests

from anubis.utils.ColorPrint import ColorPrint


def search_virustotal(self, target):
  print("Searching VirusTotal")
  headers = {'dnt':             '1', 'accept-encoding': 'gzip, deflate, br',
             'accept-language': 'en-US,en;q=0.9,it;q=0.8',
             'user-agent':      'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/62.0.3202.94 Safari/537.36',
             'accept':          'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8',
             'authority':       'www.virustotal.com',
             'cookie':          'VT_PREFERRED_LANGUAGE=en', }
  res = requests.get(
    'https://www.virustotal.com/en/domain/' + target + '/information/',
    headers=headers)
  if res.status_code == 403:
    ColorPrint.red(
      "VirusTotal is currently ratelimiting this IP - go to https://www.virustotal.com/en/domain/" + target + "/information/ and complete the captcha to continue.")
    return
  scraped = res.text
  try:
    trim_to_subdomain = scraped[
                        scraped.find("observed-subdomains"):scraped.rfind(
                          "<script>")].split('\n')
    for domain in trim_to_subdomain:
      if domain.strip().endswith("." + target):
        if domain.strip() not in self.domains and domain.endswith(target):
          self.domains.append(domain.strip())
          if self.options["--verbose"]:
            print("VirustTotal Found Domain:", domain.strip())
  except Exception as e:
    self.handle_exception(e, "Error parsing virustotal")
    pass



import requests


# TODO - implement searching google, bing, yahoo, baidu, and ask
def search_google(self):
  print("Searching Google")
  base_url = "https://google.com/search?q="
  append = "&hl=en-US&start="
  query = "site:" + self.options["TARGET"]
  for domain in self.domains:
    query += " -" + domain
  page_num = 0
  url = base_url + query + append + str(page_num)
  res = requests.get(url)



"""Packaging settings."""

from codecs import open
from os.path import abspath, dirname, join
from subprocess import call

from setuptools import Command, find_packages, setup

from anubis import __version__

this_dir = abspath(dirname(__file__))
with open(join(this_dir, 'README.md'), encoding='utf-8') as file:
  long_description = file.read()


class RunTests(Command):
  """Run all tests."""
  description = 'run tests'
  user_options = []

  def initialize_options(self):
    pass

  def finalize_options(self):
    pass

  def run(self):
    """Run all tests!"""
    errno = call(['py.test', '--cov=anubis', '--cov-report=term-missing'])
    raise SystemExit(errno)


setup(name='anubis-netsec', version=__version__,
      description='Modern and efficient subdomain enumeration and information gathering',
      long_description=long_description,
      url='https://github.com/jonluca/anubis', author='JonLuca DeCaro',
      author_email='jdecaro@usc.edu', license='GPL',
      classifiers=['Intended Audience :: Developers',
                   'Development Status :: 3 - Alpha', 'Topic :: Utilities',
                   'License :: Public Domain', 'Natural Language :: English',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python :: 3.4',
                   'Programming Language :: Python :: 3.5',
                   'Programming Language :: Python :: 3.6', ], keywords='cli',
      packages=find_packages(exclude=['docs', 'tests*']), python_requires='>=3',
      install_requires=['docopt', 'setuptools', 'SSLyze', 'python_nmap',
                        'shodan', 'requests', 'censys', 'dnspython'],
      extras_require={'test': ['coverage', 'pytest', 'pytest-cov'], },
      entry_points={'console_scripts': ['anubis=anubis.cli:main', ], },
      cmdclass={'test': RunTests},
      py_modules=["anubis.utils", "anubis.scanners"], )



# TODO - Need to implement brute forcing in a non blocking, multi threaded way
def brute_force(self, target):
  print(
    "NOT YET IMPLEMENTED - Starting brute force enumartin (warning - will take a while)")
